"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AssertOrder = void 0;
const InvalidOrder_js_1 = require("./InvalidOrder.js");
const StateMachine_js_1 = require("./StateMachine.js");
// order.atLeast(step, count) // expected to be executed at least `count` times
// order.atMost(step, count) // exected to be executed at most `count` times
class AssertOrder {
    /**
     * Gets the current expecting step.
     */
    get currentStep() {
        return this.state.step;
    }
    constructor(plan) {
        Object.defineProperty(this, "state", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.state = new StateMachine_js_1.StateMachine(plan);
    }
    /**
     * Asserts the current step is `step`.
     * @param step expected step.
     */
    is(step) {
        this.assert('is', step);
    }
    /**
     * Asserts the current step is not `step`.
     * @param step not expected step.
     */
    not(step) {
        if (this.state.isValid(step)) {
            throw new InvalidOrder_js_1.InvalidOrder(this.state.get(), 'not', [step], { ssf: this.not });
        }
    }
    /**
     * Reset to specified step.
     */
    jump(step) {
        this.state.jump(step);
    }
    /**
     * Move to next step.
     */
    move() {
        this.state.move();
    }
    /**
     * Asserts the specified step will run once.
     */
    once(step) {
        this.assert('once', step);
        this.state.move();
    }
    on(step, assert) {
        this.state.on(step, () => {
            assert(step);
        });
    }
    atLeastOnce(step) {
        const nextStep = step + 1;
        if (this.state.step === step) {
            this.state.move();
            return this.state.moveSubStep();
        }
        else if (this.state.step === nextStep && this.state.subStep > 0) {
            return this.state.moveSubStep();
        }
        else
            throw new InvalidOrder_js_1.InvalidOrder(this.state.get(), 'atLeastOnce', [step], { ssf: this.atLeastOnce });
    }
    any(steps, handler) {
        const index = steps.indexOf(this.state.step);
        if (index === -1)
            throw new InvalidOrder_js_1.InvalidOrder(this.state.get(), 'any', [steps], { ssf: this.any });
        if (handler)
            handler(steps[index]);
        this.state.move();
        return steps[index];
    }
    onAny(steps, ...asserts) {
        steps.forEach(step => {
            this.state.on(step, () => {
                let firstError;
                let hasPass = false;
                asserts.forEach(assert => {
                    try {
                        assert(step);
                        hasPass = true;
                    }
                    catch (err) {
                        if (!firstError)
                            firstError = err;
                    }
                });
                if (!hasPass)
                    throw firstError;
            });
        });
        return;
    }
    end(timeout) {
        if (timeout) {
            return new Promise(r => {
                setTimeout(r, timeout);
            }).then(() => {
                return this.end();
            });
        }
        if (!this.state.isMaxStepDefined()) {
            this.state.stopAccepting();
            return this.state.getTimeTaken();
        }
        if (this.state.isAccepting()) {
            throw new InvalidOrder_js_1.InvalidOrder(this.state.get(), 'end', [], { ssf: this.end });
        }
    }
    exactly(step, times) {
        if (this.state.isNotValid(step))
            throw new InvalidOrder_js_1.InvalidOrder(this.state.get(), 'exactly', [step, times], { ssf: this.exactly });
        if (this.state.maxSubStep === undefined) {
            this.state.maxSubStep = times;
            this.state.minSubStep = times;
        }
        return this.state.moveSubStep();
    }
    wait(step, callback) {
        if (callback)
            this.on(step, callback);
        else
            return new Promise(a => {
                this.on(step, a);
            });
    }
    assert(method, step) {
        if (this.state.isNotValid(step)) {
            throw new InvalidOrder_js_1.InvalidOrder(this.state.get(), method, [step], { ssf: this[method] });
        }
    }
}
exports.AssertOrder = AssertOrder;
//# sourceMappingURL=AssertOrder.js.map