import * as perf from 'perf_hooks';
let timeTracker;
if (global['process'] && typeof global['process'].hrtime === 'function') {
    let tick;
    timeTracker = {
        start() {
            tick = global['process'].hrtime();
        },
        taken() {
            const [second, nanoSecond] = global['process'].hrtime(tick);
            return second * 1000 + nanoSecond / 1e6;
        }
    };
}
else if (perf.performance && typeof perf.performance.now === 'function') {
    const now = perf.performance.now;
    let tick;
    timeTracker = {
        start() {
            tick = now();
        },
        taken() {
            return now() - tick;
        }
    };
}
else {
    let tick;
    timeTracker = {
        start() {
            tick = new Date().valueOf();
        },
        taken() {
            return new Date().valueOf() - tick;
        }
    };
}
export class StateMachine {
    constructor(maxStep) {
        Object.defineProperty(this, "listeners", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        Object.defineProperty(this, "step", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 1
        });
        Object.defineProperty(this, "maxStep", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "subStep", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "minSubStep", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "maxSubStep", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.maxStep = maxStep;
        timeTracker.start();
    }
    jump(step) {
        this.step = step;
        this.subStep = 0;
        return this.step;
    }
    move() {
        const listeners = this.listeners[this.step];
        this.step = this.step + 1;
        this.subStep = 0;
        if (listeners) {
            listeners.forEach(l => l());
        }
        return this.step;
    }
    moveSubStep() {
        const subStep = ++this.subStep;
        if (this.maxSubStep === this.subStep)
            this.move();
        return subStep;
    }
    get() {
        const { step, subStep, maxStep, minSubStep, maxSubStep } = this;
        return {
            step,
            maxStep,
            subStep,
            minSubStep,
            maxSubStep
        };
    }
    on(step, listener) {
        this.listeners[step] = [listener];
    }
    isNotValid(step) {
        return step !== this.step || (this.maxStep !== undefined && this.maxStep < step);
    }
    isValid(step) {
        return step === this.step;
    }
    isMaxStepDefined() {
        return this.maxStep !== undefined;
    }
    stopAccepting() {
        this.maxStep = this.step - 1;
    }
    isAccepting() {
        // istanbul ignore next
        return this.maxStep ? this.maxStep >= this.step : true;
    }
    getTimeTaken() {
        return timeTracker.taken();
    }
}
//# sourceMappingURL=StateMachine.js.map